//******************************************************************
//* SERVO BASIC CNC DRIVER TUTORIAL CODE
//* VERSION 1.0
//* COMPILED WITH CCS C 4.097
//* WRITTEN BY FRED EADY FOR SERVO MAGAZINE
//* LAST UPDATED 08/25/2009
//* CHANGES/ADDITIONS
//******************************************************************

#include "C:\servo-ccs-cnc\servo-ccs-cnc.h"
//******************************************************************
//* CCS C CONFIGURATION
//******************************************************************
#use fast_io(ALL)
//******************************************************************
//* FUNCTION PROTOTYPES
//******************************************************************
void init(void);
int8 actuator_position(void);
//******************************************************************
//* SENSOR I/O DEFINITIONS
//******************************************************************
#define SENSOR_MIN   !input(PIN_A0)   
#define SENSOR_MAX   !input(PIN_A1)
#define FAULT      0   
#define MAX         1   
#define MIN         2   
#define STUCK      3
#define DRIVE_MIN   PIN_C4
#define DRIVE_MAX   PIN_C5
//******************************************************************
//* CCS LCD DEFINITIONS
//******************************************************************
#define LCD_ENABLE_PIN    PIN_E0
#define LCD_RS_PIN       PIN_E2
#define LCD_RW_PIN       PIN_E1
#define LCD_TYPE       2   //2 line display
#include <lcd.c>
//******************************************************************
//* ACTUATOR POSITION FUNCTION
//* RETURNS POSITION OF ACTUATOR
//******************************************************************
int8 actuator_position(void)
{
   int8 position;
   position = STUCK;
   if(SENSOR_MIN && !SENSOR_MAX)
   {
      //LINEAR ACTUATOR AT MINIMUM STROKE CODE GOES HERE
      position = MIN;
   }
   else if(!SENSOR_MIN && SENSOR_MAX)
   {
      //LINEAR ACTUATOR AT MAXIMUM STROKE CODE GOES HERE
      position = MAX;
   }
   else if(SENSOR_MIN && SENSOR_MAX)
   {
      position = FAULT;
   }
   return position;
}
//******************************************************************
//* INITIALIZE FUNCTION
//******************************************************************
void init(void)
{
   setup_adc_ports(NO_ANALOGS|VSS_VDD);
   setup_adc(ADC_OFF|ADC_TAD_MUL_0);
   setup_psp(PSP_DISABLED);
   setup_spi(SPI_SS_DISABLED);
   setup_wdt(WDT_OFF);
   setup_timer_0(RTCC_OFF);
   setup_timer_1(T1_DISABLED);
   setup_timer_2(T2_DISABLED,0,1);
   setup_timer_3(T3_DISABLED|T3_DIV_BY_1);
   setup_comparator(NC_NC_NC_NC);
   setup_vref(FALSE);
   set_tris_a(0b11111111);   
   set_tris_b(0b11111111);
   set_tris_c(0b11001111);
   set_tris_d(0b11110000);
   set_tris_e(0b11111000);
} 
//******************************************************************
//* MAIN PROGRAM FUNCTION
//******************************************************************
void main()
{
   int8 rev_cntr;
   
      init();
      lcd_init();
   lcd_putc("\f STATUS");
   output_bit(DRIVE_MAX,1);
   output_bit(DRIVE_MIN,1);
   rev_cntr = 0;
   
   //SENSOR EMULATOR
   do{
      switch(rev_cntr)
      {
         case 0://BOTH SENSORS ON
            output_bit(DRIVE_MAX,0);
            output_bit(DRIVE_MIN,0);
            ++rev_cntr;
         break;
         case 1://MAX SENSOR ON
            output_bit(DRIVE_MAX,0);
            output_bit(DRIVE_MIN,1);
            ++rev_cntr;
         break;
         case 2://MIN SENSOR ON
            output_bit(DRIVE_MAX,1);
            output_bit(DRIVE_MIN,0);
            ++rev_cntr;
         break;
         case 3://BOTH SENSORS OFF
            output_bit(DRIVE_MAX,1);
            output_bit(DRIVE_MIN,1);
            rev_cntr = 0;
         break;
      }
      
      //MAIN CODE LOOP   
      switch (actuator_position())
      {
         case MIN:
            lcd_putc("\nACTU=MIN");
         break;
         case MAX:
            lcd_putc("\nACTU=MAX");
         break;
         case STUCK:
            lcd_putc("\n STUCK  ");
         break;
         case FAULT:
            lcd_putc("\n FAULT  ");
         break;
      }
         
      delay_ms(2000);
      
   }while(1);

}
